!
! rotorlevels --- compute the cross correlation between the aligning
!       laser pulse and the probing x-ray pulse
! Christian Buth
! 24 Apr 2007
!
! Version V1.0.0: 24 Apr 2007 (Symmetric rotor Hamiltonian)
!


      program rotorlevels
      use mathtl
      use physconst
      implicit none

      integer Jmax, J, K, i

      real(dblprec) EJK, RotConstA, RotConstB, Z, beta, temperature, T_J, E_J, av_T

      ! Initialization
      write (*,*) 'rotorlevels V1.0.0, energy levels of a ridgid rotor'
      write (*,*) '(c) 2007 Christian Buth'
      write (*,*)

      Jmax = 5
      RotConstA = 0.191205D0
      RotConstB = 0.069834D0

      ! Calculate energy levels of a symmetric rotor
      write (*,*) 'E_JK = B * J*(J+1) + (A-B) * K**2'
      write (*,*) 'A =', RotConstA, 'cm^-1, B =', RotConstB, 'cm^-1'
      write (*,*)
      do J=0, Jmax
        write (*,'(2A8,A20)') 'J', '|K|', 'E_JK [cm^-1]'
        do K=0, J
          EJK = RotConstB * real(J*(J+1), dblprec) + (RotConstA - RotConstB) * real(K**2, dblprec)
          write (*,'(2I8,F20.6)') J, K, EJK
        end do
        write (*,*)
      end do

      ! Calculate average rotational temperature
      ! (linear rotor approximation; CF3Br model statistical weights all unity)
      ! Partition sum
      RotConstA = 0.0D0
      RotConstB = 0.069834D0 * invCentimeterToHartree
      Jmax = 1000

      do i=1, 20
        temperature = 1.0D0 * real(i, dblprec) * KelvinToHartree
        beta = 1.0D0 / temperature
        Z = 0.0D0
        do J=1, Jmax
          E_J = RotConstB * real(J*(J+1), dblprec)
          Z = Z + real(2*J+1, dblprec) * exp(-beta * E_J)
        end do
        ! Thermal average
        av_T = 0.0D0
        do J=1, Jmax
          E_J = RotConstB * real(J*(J+1), dblprec)
          T_J = M_PI / RotConstB / sqrt(real(J*(J+1), dblprec))
          if (i == 20) then
            write (100,*) J, T_J * t0ToPicosecond, real(2*J+1, dblprec) * exp(-beta * E_J)
          end if
          av_T = av_T + T_J * real(2*J+1, dblprec) * exp(-beta * E_J)
        end do
        av_T = av_T / Z
        av_T = av_T * t0ToPicosecond
        write (*,*) 'Temperature ', temperature * HartreeToKelvin, 'K, averaged rotational period ', av_T, ' ps'
      end do

      T_J = 1.0D0 / (2.0D0 * RotConstB)
      T_J = T_J * 2.0D0 * M_PI * t0ToPicosecond
      write (*,*)
      write (*,*) 'Characteristic rotational period 2B=', T_J, ' ps'

      ! Clean up and exit gracefully
      write (*,*)
      write (*,*) 'Analysis done!'
      end program rotorlevels
