!
! fulldiagfloquet --- carry out a Floquet-style calculation of the atom
!       in the presence of a laser field diagonalizing the matrix fully
! Christian Buth
! 15 June 2006
!
! Version 1.0.0: 23 June 2005 (Integrated thetaeval to diagonalize the
!       Hamiltonian plus smooth exterior scaling CAP)
!


      subroutine fulldiagfloquet(jobz, dimen, ceig, cvec)
      use matrixfile
      use floquetmat, fulldiagfloquetdbl => fulldiagfloquet
      implicit none

      character, intent(in) :: jobz
      integer, intent(in) :: dimen
      complex(dblprec), intent(out) :: ceig(dimen)
      complex(dblprec), optional, intent(out) :: cvec(dimen,dimen)

      real(dblprec) rwork(2*dimen)
      complex(dblprec) cwork(30*dimen)
      complex(dblprec) zHmat(dimen,dimen)

      logical mfeof
      integer i, j, ldvl, info
      complex(dblprec) vl, norm
      type(tmatrixfile) mf
      complex(dblprec) zdotu
      external zdotu, zscal, zgeev

      ! Parameter check
      if ((jobz == 'v' .or. jobz == 'V') .and. (.not. present(cvec))) then
        stop 'fulldiagfloquet: eigenvectors need to be stored somewhere!'
      end if

      ! Read zHmat from disc
      mf%readfile = .true.
      mf%fmtxcom  = 393
      mf%fmtxdia  = 394
      mf%fmtxoff  = 395
      call openmatrixfile(mf)

      ! Get diagonal elements of the matrix
      zHmat = 0
      do
        call readmatrixbuffer(mf, .true., mfeof)
        if (mfeof) then
          exit
        end if

        do i=1, mf%nwd
          zHmat(mf%idx(i), mf%idx(i)) = mf%diag(i)
        end do
      end do

      ! Get off-diagonal elements of the matrix
      do
        call readmatrixbuffer(mf, .false., mfeof)
        if (mfeof) then
          exit
        end if

        do i=1, mf%nwo
          zHmat(mf%iox(i), mf%jox(i)) = mf%off(i)
          zHmat(mf%jox(i), mf%iox(i)) = mf%off(i)
        end do
      end do

      call closematrixfile(mf)

      write(*,*) 'Diagonalizing the complex symmetric Hamiltonian ', &
     &           'with dimension =', dimen

      ! Diagonalize the complex symmetric matrix zHmat
      ldvl = 1
      call zgeev('N',jobz,dimen,zHmat,dimen,ceig,vl,ldvl,cvec,dimen, &
     &          cwork,30*dimen,rwork,info)

      if (info /= 0) then
        write (*,*) 'info =', info
        stop 'thetaeval: zgeev info /= 0'
      end if

      ! Nomalize all eigenvectors to unity
      if (jobz == 'v' .or. jobz == 'V') then
        do j = 1, dimen
          norm = zdotu(dimen,cvec(1,j),1,cvec(1,j),1)
          norm = (1.0D0,0.0D0) / sqrt(norm)
          call zscal(dimen,norm,cvec(1,j),1)
        end do
      end if
      end subroutine fulldiagfloquet
